#!/bin/bash
if [ $# -eq 0 ]; then
cat << EOF
Usage: configure.sh <OpenCV prefix> [norpath]
e.g. # ./configure.sh /usr

If you use the 'norpath' option, files will be built without
hard-coded rpath (useful for distro packaging).

Build dependencies:
 * cmake >= 2.6
 * boost >= 1.35 (program_options, filesystem)
 * Qt >= 4.7
 * OpenCV >= 2.3

Use 'ccmake .' for further refinement of the configuration. See 'man cmake'.
EOF
exit 1
fi

if [ $2 = norpath ]; then
	skiptxt='-DCMAKE_SKIP_RPATH=on'
else
	skiptxt=''
fi

cmake -DOpenCV_DIR="$1/share/OpenCV" \
-DCMAKE_BUILD_TYPE=Release \
$skiptxt .
cmake \
-DGerbil_Viewer=on \
-DGerbil_Edge_Detect=on \
-DVole_Seg_Graphs=on \
-DVole_Seg_MeanShift=on \
-DVole_Shell=on \
.
if [ $? -ne 0 ]; then
echo " *** Please use CMake to resolve configuration issues (e.g. 'ccmake .')."
fi
